#=BankValInternational
#
#==Unified Software
#
#Module containing classes to handle calls to Unified Sotware's
#BankValInternational web services
#
#*[BankValInternational::GetIBAN] Which handles calls to _IBANValidate_
#
#*[BankValInternational::GetSWIFT] Which handles calls to <i>GetBankDetails2</i>
#
#*[BankValInternational::GetABA] Which handles calls to _getABAdetails_
#
module BankValInternational
  require 'net/https'
  require 'uri'

  #=BankValInternational::GetIban
  #
  #Class to control calls to _IBANValidate_
  #
  #=Synopsis
  #
  # requires 'BankValInternational'
  #
  # @obj1 = BankValInternational::GetIBAN.new
  # @ans = obj1.bankval_int_ibanval('json','AN_IBAN_NUMBER','userID','PINNo')
  #
  class BankValInternational::GetIBAN
    @userid
    @pin
    @iban
    @format
    @service_url
    
    #Method to control calls to _IBANValidate_
    #
    #==Parameters
    #1.Return Format:: must be either 'json', 'xml' or 'csv'
    #2.Iban Number:: the number for validation
    #3.UserId:: can obtained from http://www.unifiedsoftware.co.uk/freetrial/free-trial-home.html
    #4.PIN:: can obtained from http://www.unifiedsoftware.co.uk/freetrial/free-trial-home.html
    #
    #The parameters *must* be passed in the order shown above
    #
    #==Returns
    #The response from the webservice is returned as a string in either:
    #*XML format
    #*json format
    #*csv format
    #depending on the Return Format Parameter passed in
    #
    def bankval_int_ibanval(*args)
      @format = args[0].downcase
      @iban = args[1]
      @userid = args[2]
      @pin = args[3]
      if args.size !=4
        return "ERROR - not enough parameters supplied"
      end
      format_validation
      if @error_string != nil
        format_error_string
        return @error_string
      end
      build_req_url
      return "#{BankValInternational::GoValidate.new.validate(@service_url)}"
    end
    
    private
    
    #method to build the base request url for the REST call
    def build_req_url()
      @service_url = "bankvalint/ibanvalidator/userid/#{@userid}/pin/#{@pin}/iban/#{@iban}/#{@format}/"
    end
    def format_validation
      @error_string = nil
      if @format !~  /^json$|^xml$|^csv$/
        @error_string = "INVALID - Result Format"
      elsif @iban !~ /^[A-Z,0-9]{1,34}$/
        @error_string = "INVALID - FORMAT"
      elsif @pin !~ /^[0-9]{5}$/
        @error_string = "ERROR - Invalid User ID/PIN"
      elsif @userid !~ /^[a-zA-Z\-_][a-zA-Z][a-zA-Z]*\D\d\d\d$/
        @error_string = "ERROR - Invalid User ID/PIN"
      end
    end

    def format_error_string()
      if @format == 'xml'
        @error_string = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><iban><result>" + @error_string + "</result></iban>"
      elsif @format == 'json'
        @error_string = "{\"result\":\"" + @error_string + "\"}"
      end
    end

  end

  #=BankValInternational::GetSWIFT
  #
  #Class to control calls to <i>getBankDetails2</i>
  #
  #=Synopsis
  #
  # requires 'BankValInternational'
  #
  # @obj1 = BankValInternational::GetSWIFT.new
  # @ans = obj1.bankval_int_get_swift('xml','A_SWIFT_BIC','userID','PINNo')
  #
  class BankValInternational::GetSWIFT
    @userid
    @pin
    @swiftbic
    @format
    @service_url

    #Method to control calls to _getBankDetails2_
    #
    #==Parameters
    #1.Return Format:: must be either 'json', 'xml' or 'csv'
    #2.Swift BIC Number:: the number for validation
    #3.UserId:: can obtained from http://www.unifiedsoftware.co.uk/freetrial/free-trial-home.html
    #4.PIN:: can obtained from http://www.unifiedsoftware.co.uk/freetrial/free-trial-home.html
    #
    #The parameters *must* be passed in the order shown above
    #
    #==Returns
    #The response from the webservice is returned as a string in either:
    #*XML format
    #*json format
    #*csv format
    #depending on the Return Format Parameter passed in
    #
    def bankval_int_getswift(*args)
      @format = args[0].downcase
      @swiftbic = args[1]
      @userid = args[2]
      @pin = args[3]
      if args.size !=4
        return "ERROR - not enough parameters supplied"
      end
      format_validation
      if @error_string != nil
        format_error_string
        return @error_string
      end
      build_req_url
      return "#{BankValInternational::GoValidate.new.validate(@service_url)}"
    end

    private

    #method to build the base request url for the REST call
    def build_req_url()
      @service_url = "bankvalint/bankdetails2/userid/#{@userid}/pin/#{@pin}/swiftbic/#{@swiftbic}/#{@format}/"
    end
    def format_validation
      @error_string = nil
      if @format !~  /^json$|^xml$|^csv$/
        @error_string = "INVALID - Result Format"
      elsif @swiftbic !~ /^[A-Z,0-9]{8}$/
        @error_string = "INVALID"
      elsif @pin !~ /^[0-9]{5}$/
        @error_string = "ERROR - Invalid User ID/PIN"
      elsif @userid !~ /^[a-zA-Z\-_][a-zA-Z][a-zA-Z]*\D\d\d\d$/
        @error_string = "ERROR - Invalid User ID/PIN"
      end
    end

    def format_error_string()
      if @format == 'xml'
        @error_string = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><swiftbic><result>" + @error_string + "</result><bic></bic><name1></name1><name2></name2><name3></name3><address1></address1><address2></address2><address3></address3><address4></address4><location></location><country></country></swiftbic>"
      elsif @format == 'json'
        @error_string = "{\"result\":\"" + @error_string + "\",\"bic\":\"\",\"name1\":\"\",\"name2\":\"\",\"name3\":\"\",\"address1\":\"\",\"address2\":\"\",\"address3\":\"\",\"address4\":\"\",\"location\":\"\",\"country\":\"\"}"
      end
    end
    
  end

  #=BankValInternational::GetABA
  #
  #Class to control calls to _getABAdetails_
  #
  #=Synopsis
  #
  # requires 'BankValInternational'
  #
  # @obj1 = BankValInternational::GetABA.new
  # @ans = obj1.bankval_int_ibanval('json','AN_ABA','userID','PINNo')
  #
  class BankValInternational::GetABA
    @userid
    @pin
    @aba
    @format
    @service_url

    #Method to control calls to _getABAdetails_
    #
    #==Parameters
    #1.Return Format:: must be either 'json', 'xml' or 'csv'
    #2.ABA Number:: the number for validation
    #3.UserId:: can obtained from http://www.unifiedsoftware.co.uk/freetrial/free-trial-home.html
    #4.PIN:: can obtained from http://www.unifiedsoftware.co.uk/freetrial/free-trial-home.html
    #
    #The parameters *must* be passed in the order shown above
    #
    #==Returns
    #The response from the webservice is returned as a string in either:
    #*XML format
    #*json format
    #*csv format
    #depending on the Return Format Parameter passed in
    #
    def bankval_int_getaba(*args)
      @format = args[0].downcase
      @aba = args[1]
      @userid = args[2]
      @pin = args[3]
      if args.size !=4
        return "ERROR - not enough parameters supplied"
      end
      format_validation
      if @error_string != nil
        format_error_string
        return @error_string
      end
      build_req_url
      return "#{BankValInternational::GoValidate.new.validate(@service_url)}"
    end

    private

    #method to build the base request url for the REST call
    def build_req_url()
      @service_url = "bankvalint/abadetails/userid/#{@userid}/pin/#{@pin}/aba/#{@aba}/#{@format}/"
    end

    def format_validation()
      @error_string = nil
      if @format !~  /^json$|^xml$|^csv$/
        @error_string = "INVALID - Result Format"
      elsif @aba !~ /^[0-9]{9}$/
        @error_string = "INVALID"
      elsif @pin !~ /^[0-9]{5}$/
        @error_string = "ERROR - Invalid User ID/PIN"
      elsif @userid !~ /^[a-zA-Z\-_][a-zA-Z][a-zA-Z]*\D\d\d\d$/
        @error_string = "ERROR - Invalid User ID/PIN"
      end
    end

    def format_error_string()
      if @format == 'xml'
        @error_string = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><aba><result>" + @error_string + "</result><routingnumber></routingnumber><bankname></bankname><bankaddress></bankaddress><city></city><state></state><zipcode></zipcode><telephone></telephone><areacode></areacode><fips></fips><timezone></timezone><dst></dst><latitude></latitude><longitude></longitude><msa></msa><pmsa></pmsa><county></county></aba>"
      elsif @format == 'json'
        @error_string = "{\"result\":\"" + @error_string + "\",\"routingnumber\":\"\",\"bankname\":\"\",\"bankaddress\":\"\",\"city\":\"\",\"state\":\"\",\"zipcode\":\"\",\"telephone\":\"\",\"areacode\":\"\",\"fips\":\"\",\"timezone\":\"\",\"dst\":\"\",\"latitude\":\"\",\"longitude\":\"\",\"msa\":\"\",\"pmsa\":\"\",\"county\":\"\"}"
      end
    end
  end

  #=BankValInternational::GoValidate
  #
  #Class to make calls to Unified Softwares web services
  #
  #
  class BankValInternational::GoValidate
    @base_url

    #Method to make REST web service calls and return response
    #if call to the main data centre fails the back up method is called
    #to make a call to the back up data centre
    #
    #==Parameters
    #1. Serv_url is the services part of the URL for the REST call
    #
    #==Returns
    #The response from the web service in requested format (XML,JSON or CSV)
    #
    #
    def validate(ser_url)
      @base_url = "https://www.unifiedsoftware.co.uk/services/"
      uriobj = URI.parse(@base_url)
      full_path = uriobj.path + ser_url
      conn = Net::HTTP.new(uriobj.host, uriobj.port)
      conn.use_ssl = true
      conn.verify_mode = OpenSSL::SSL::VERIFY_NONE
      begin
        webresponse = conn.get2(URI.encode(full_path))
      rescue Exception => err
        webresponse = failover(ser_url)
      end
      if webresponse == nil
        return "Unknown Error-Check Local Network"
      end
      return webresponse.body
    end

    #Method to make backup REST web service calls and return response
    #if call to the main data centre fails this method is called
    #to make a call to the back up data centre
    #
    #==Parameters
    #1. Serv_url is the services part of the URL for the REST call
    #
    #==Returns
    #The response from the web service as a string in requested format (XML,JSON or CSV)
    #or..
    #error message in case of error
    #
    def failover(ser_url)
      @base_url = "https://www.unifiedservices.co.uk/services/"
      uriobj = URI.parse(@base_url)
      full_path = uriobj.path + ser_url
      conn = Net::HTTP.new(uriobj.host, uriobj.port)
      conn.use_ssl = true
      conn.verify_mode = OpenSSL::SSL::VERIFY_NONE
      begin
        webresponse = conn.get2(URI.encode(full_path))
      rescue Exception => err
        return "Network Error" + err
      end
      return webresponse
    end
  end

end
